/*
 * Project: MoleCuilder
 * Description: creates and alters molecular systems
 * Copyright (C)  2012 University of Bonn. All rights reserved.
 * Please see the COPYING file or "Copyright notice" in builder.cpp for details.
 * 
 *
 *   This file is part of MoleCuilder.
 *
 *    MoleCuilder is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    MoleCuilder is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with MoleCuilder.  If not, see <http://www.gnu.org/licenses/>. 
 */

/*
 * PairPotential_Harmonic.cpp
 *
 *  Created on: Sep 26, 2012
 *      Author: heber
 */


// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "CodePatterns/MemDebug.hpp"

#include "PairPotential_Harmonic.hpp"

#include "CodePatterns/Assert.hpp"

#include "Potentials/helpers.hpp"

PairPotential_Harmonic::results_t
PairPotential_Harmonic::operator()(
    const arguments_t &arguments
    ) const
{
  ASSERT( arguments.size() == 1,
      "PairPotential_Harmonic::operator() - requires exactly one argument.");
  const argument_t &r_ij = arguments[0];
  const result_t result = spring_constant * Helpers::pow( r_ij.distance - equilibrium_distance, 2 );
  return std::vector<result_t>(1, result);
}

PairPotential_Harmonic::derivative_components_t
PairPotential_Harmonic::derivative(
    const arguments_t &arguments
    ) const
{
  ASSERT( arguments.size() == 1,
      "PairPotential_Harmonic::operator() - requires exactly one argument.");
  derivative_components_t result;
  const argument_t &r_ij = arguments[0];
  result.push_back( 2. * spring_constant * ( r_ij.distance - equilibrium_distance) );
  ASSERT( result.size() == 1,
      "PairPotential_Harmonic::operator() - we did not create exactly one component.");
  return result;
}

void PairPotential_Harmonic::setParameters(const parameters_t &_params)
{
  ASSERT( _params.size() == 2,
      "PairPotential_Harmonic::setParameters() - we need exactly two parameters.");
  spring_constant = _params[0];
  equilibrium_distance = _params[1];
}

PairPotential_Harmonic::parameters_t PairPotential_Harmonic::getParameters() const
{
  parameters_t params(2, 0.);
  params[0] = spring_constant;
  params[1] = equilibrium_distance;
  return params;
}
